#!/usr/bin/python3

# Rev A.01.08 - May 9, 2023
# Rev A.01.09 - May 28, 2023 - added nrun var
# Rev A.01.10 - September 16, 2024 - added squelch, agc vars
# The DZ Company, LLC
# This software may be distributed freely as long as credit is given to DZKit

import queue
import numpy as np
import time
from scipy.signal import ellip
from stream_sdr_to_audio import Filter_coeff

# sw_rx_globals.py: Globals for Project sw_rx.py
keyer_mode = 1
nrun = 0
cmd = 0
data = 0
hs_mode = 0
afg = 300
prev_afg = 700
agc = "Slow"
squelch = -25
save_agc_button = "       "
lna = 2
gr = 12		# when lna = 2, gr = 12
rfg = 512	# init rfg to midpoint
rpg = 256
p = 0
stream = 0
rxStream = 0
s_ready = False
dBm = -50
t0 = time.perf_counter()
t1 = time.perf_counter()
volume = 100
stopped = False
new_buf_avail = False
dBm = 999
ly = 0
iter_nr = 0
maxi_nr = 0
maxi_buf = [0.0, 0.0, 0.0, 0.0, 0.0]
smx1 = 790
smy1 = 490
smx2 = 951
smy2 = 500
smtr_enabled = True

canvas1 = 0
seglit = 0
seg1 = 0
seg2 = 0
seg3 = 0
seg4 = 0
seg5 = 0
seg6 = 0
seg7 = 0
seg8 = 0
seg9 = 0
seg10 = 0
seg11 = 0
seg12 = 0
seg13 = 0
seg14 = 0
seg15 = 0
seg16 = 0

# GUI scale factors for display (1024 x 600 vs orig 800 x 480)
xscale = 1.28
yscale = 1.25
xband_offset = 45

page = 1

# Select mode, frequency and sample rate
mode = "AM"
center_freq = 600e3
currband_start = 200e3
currband_span = 1.8e6
freq_inc = 10000
sample_rate = 250e3
max_buffer_size = 8064
num_rx_calls = 4 
num_samps = max_buffer_size * num_rx_calls
decimation = 8
min = 0
ptp = 0
avg = 0

audio_buf = queue.Queue(maxsize=0)
audio_stream_started = False
drate = int(sample_rate / decimation)
framesize = int(num_samps / decimation)
ab_frame = np.array([0]*framesize, np.int16) 

sample_buffer = np.array([0]*max_buffer_size, np.complex64)
x = np.array([0]*num_samps, np.complex64)
z = np.array([0]*num_samps, np.complex64)
last_buf = np.array([0]*num_samps, np.complex64)

fmf = Filter_coeff(0,0)
bpf800 = Filter_coeff(0,0)
bpf2r8k = Filter_coeff(0,0)
bpf6k = Filter_coeff(0,0)
bpf20k = Filter_coeff(0,0)
exp_jwt_3k = []
exp_jwt_1r4k = []
exp_jwt_m1r4k = []
exp_jwt_400 = []
exp_jwt_m400 = []
frq_ofs = 0

line = 0	
orangeline = np.array([0,0,0,0,0,0,0,0])
lF1 = 0
freq_box_exists = False

# Band info: start freq of band, last frequency used on that band, mode
# dial pointer position is calculated from current frequency
selected_band_p1 = 0
selected_band_p2 = 0
selected_band_p3 = 0
selected_band_p4 = 0
selected_band_p5 = 0
selected_band_p6 = 0
selected_band_p7 = 0
selected_band_p8 = 0
selected_band_p9 = 0
selected_band_p10 = 0

# top of dial pointers for each band
ytop = np.array([
				25*yscale-7,
				75*yscale-7,	
				125*yscale-7,	
				175*yscale-7,
				225*yscale-7,	
				275*yscale-7,
				325*yscale-7,	
				375*yscale-7], 
				dtype = np.int16)

band_info_p1 = np.array(
				   [(200e3,600e3,"AM"),
					(2e6,3.5e6,"LSB"),
					(5e6,7.0e6,"LSB"),
					(8e6,10.0e6,"AM"),
					(11e6,12.5e6,"AM"),
					(14e6,14.0e6,"USB"),
					(17e6,18.068e6,"USB"),
					(20e6,21.0e6,"USB")],
			 dtype =[('start_f', '<f4'),
					 ('last_f', '<f4'),
					 ('mode', '<U7')])

band_info_p2 = np.array(
				   [(23e6,24.89e6,"USB"),
					(26e6,28.0e6,"USB"),
					(29e6,29.0e6,"NBFM"),
					(50e6,50.0e6,"USB"),
					(53e6,53.0e6,"USB"),
					(75e6,75.0e6,"FM"),
					(78e6,78.0e6,"FM"),
					(81e6,81.0e6,"FM")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])

band_info_p3 = np.array(
				   [(84e6,84e6,"FM"),
					(87e6,88.1e6,"FM"),
					(90e6,90.1e6,"FM"),
					(93e6,95.5e6,"FM"),
					(96e6,98.1e6,"FM"),
					(99e6,101.1e6,"FM"),
					(102e6,102.5e6,"FM"),
					(105e6,105.1e6,"FM")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])
					
band_info_p4 = np.array(
				   [(108e6,108e6,"AM"),
					(111e6,111e6,"AM"),
					(114e6,114e6,"AM"),
					(117e6,117e6,"AM"),
					(120e6,120e6,"AM"),
					(123e6,123e6,"AM"),
					(126e6,126e6,"AM"),
					(129e6,129e6,"AM")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])
					
band_info_p5 = np.array(
				   [(132e6,132e6,"AM"),
					(135e6,135e6,"AM"),
					(138e6,138e6,"AM"),
					(141e6,141e6,"AM"),
					(144e6,144e6,"NBFM"),
					(147e6,147e6,"NBFM"),
					(150e6,150e6,"NBFM"),
					(153e6,153e6,"NBFM")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])
					
band_info_p6 = np.array(
				   [(420e6,420e6,"NBFM"),
					(423e6,423e6,"NBFM"),
					(426e6,426e6,"NBFM"),
					(429e6,429e6,"NBFM"),
					(432e6,432e6,"NBFM"),
					(435e6,435e6,"NBFM"),
					(438e6,438e6,"NBFM"),
					(441e6,441e6,"NBFM")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])
					
band_info_p7 = np.array(
				   [(444e6,444e6,"NBFM"),
					(447e6,447e6,"NBFM"),
					(450e6,450e6,"NBFM"),
					(453e6,453e6,"NBFM"),
					(456e6,456e6,"NBFM"),
					(459e6,459e6,"NBFM"),
					(462e6,462e6,"NBFM"),
					(465e6,465e6,"NBFM")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])
					
band_info_p8 = np.array(
				   [(1240e6,1240e6,"USB"),
					(1243e6,1243e6,"USB"),
					(1246e6,1246e6,"USB"),
					(1249e6,1249e6,"USB"),
					(1252e6,1252e6,"USB"),
					(1255e6,1255e6,"USB"),
					(1258e6,1258e6,"USB"),
					(1261e6,1261e6,"USB")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])
					
band_info_p9 = np.array(
				   [(1264e6,1264e6,"USB"),
					(1267e6,1267e6,"USB"),
					(1270e6,1270e6,"USB"),
					(1273e6,1273e6,"USB"),
					(1276e6,1276e6,"USB"),
					(1279e6,1279e6,"USB"),
					(1282e6,1282e6,"USB"),
					(1285e6,1285e6,"USB")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])
					
band_info_p10 = np.array(
				   [(1288e6,1288e6,"USB"),
					(1291e6,1291e6,"USB"),
					(1294e6,1294e6,"USB"),
					(1297e6,1297e6,"USB"),
					(1300e6,1300e6,"USB"),
					(1303e6,1303e6,"USB"),
					(1306e6,1306e6,"USB"),
					(1309e6,1309e6,"USB")],
             dtype=[('start_f', '<f4'),
					('last_f', '<f4'),
					('mode', '<U7')])

CB = [	26965, 26975, 26985, 27005, 27015, 27025, 27035, 27055,
		27065, 27075, 27085, 27105, 27115, 27125, 27135, 27155,
		27165, 27175, 27185, 27205, 27215, 27225, 27255, 27235,
		27245, 27265, 27275, 27285, 27295, 27305, 27315, 27325,
		27335, 27345, 27355, 27365, 27375, 27385, 27395, 27405	] 
				
CAP = 26620

BIZRAD = [27430, 27450, 27470, 27490, 27510, 27530]

MARINED1 = 12290
MARINE1 = [12353, 12356, 12359, 12362, 12365]

MARINED2 = 16420
MARINE2 = [16528, 16531, 16534, 16537, 16540]

MARINE3 = [18825, 18828, 18831, 18834, 18837, 18840, 18843]

MARINE4 = [22159, 22162, 22165, 22168, 22171, 22174, 22177]

MARINE5 = [25100, 25103, 25106, 25109, 25112, 25115, 25118]

def get_freq_name():
	f = center_freq/1e3
	if f in CB:
		name = "CH " + str(1 + CB.index(f))
	elif f == CAP:
		name = "CAP"
	elif f in BIZRAD:
		name = "BRS CH " + str(1 + BIZRAD.index(f))
	elif f == MARINED1 or f == MARINED2:
		name = "MARINE DISTRESS CALLING"
	elif f in MARINE1: 
		name = "MARINE CH " + chr(ord("A") + MARINE1.index(f))
	elif f in MARINE2:
		name = "MARINE CH " + chr(ord("A") + MARINE2.index(f))
	elif f in MARINE3:
		name = "MARINE CH " + chr(ord("A") + MARINE3.index(f))
	elif f in MARINE4:
		name = "MARINE CH " + chr(ord("A") + MARINE4.index(f))
	elif f in MARINE5:
		name = "MARINE CH " + chr(ord("A") + MARINE5.index(f))
	else:
		name = -1
	return name
